@@ -1,14 +1,16 @@
 """Domain-level service to index data related to a provider."""
 
+import logging
+import time
 from datetime import datetime, timezone
 from typing import Optional
 
 import daiquiri
 from dbnomics_data_model import DatasetCode
 from dbnomics_data_model.storage.adapters.filesystem import FileSystemStorage
-from dbnomics_data_model.storage.errors import StorageError
-from humanfriendly import pluralize
-from tenacity import Retrying, stop_after_attempt, wait_random_exponential
+from humanfriendly import format_timespan
+from humanfriendly.text import pluralize
+from tenacity import Retrying, after_log, retry_if_not_exception_type, stop_after_attempt, wait_random_exponential
 
 from dbnomics_solr.dbnomics_solr_client import DatasetAlreadyIndexed, DBnomicsSolrClient, format_date_for_solr
 
@@ -40,6 +42,12 @@ def index_provider(
     provider_code = provider_metadata.code
     logger.debug("About to index data for provider %r", provider_code)
 
+    logger.info("Indexing provider metadata...", provider_code=provider_code)
+    dbnomics_solr_client.index_provider(provider_metadata)
+    logger.debug("Committing Solr document for provider metadata...")
+    dbnomics_solr_client.commit()
+    logger.info("Provider metadata indexed", provider_code=provider_code)
+
     # Index datasets
 
     if dataset_codes is None:
@@ -59,13 +67,18 @@ def index_provider(
     skipped_dataset_codes = set()
 
     for dataset_index, dataset_code in enumerate(dataset_codes, start=1):
+        t0 = time.time()
         logger.debug("About to index dataset %r (%d/%d)", dataset_code, dataset_index, len(dataset_codes))
 
         # Sometimes Solr is too busy and responds an error, so add a retry strategy
-        # instead of failing the whole dataset indexation.
+        # instead of failing the dataset indexation.
         try:
             for attempt in Retrying(
-                reraise=True, stop=stop_after_attempt(3), wait=wait_random_exponential(multiplier=1, max=60)
+                after=after_log(logger, logging.ERROR),
+                reraise=True,
+                retry=retry_if_not_exception_type(DatasetAlreadyIndexed),
+                stop=stop_after_attempt(3),
+                wait=wait_random_exponential(multiplier=1, max=60 * 5),
             ):
                 with attempt:
                     dbnomics_solr_client.index_dataset(
@@ -73,7 +86,7 @@ def index_provider(
                     )
         except DatasetAlreadyIndexed as exc:
             logger.debug(
-                "Dataset %r (%d/%d) is already indexed with dir hash %r "
+                "Dataset %r (%d/%d) is already indexed with the directory hash %r "
                 "and was not updated since previous indexation, skipping dataset",
                 exc.dataset_code,
                 dataset_index,
@@ -81,7 +94,7 @@ def index_provider(
                 exc.dir_hash,
             )
             continue
-        except StorageError:
+        except Exception:
             logger.exception("Error indexing dataset %r, skipping dataset", dataset_code)
             skipped_dataset_codes.add(dataset_code)
             continue
@@ -90,13 +103,13 @@ def index_provider(
 
         logger.debug("Committing Solr documents for dataset %r...", dataset_code)
         dbnomics_solr_client.commit()
-        logger.info("Dataset %r (%d/%d) indexed", dataset_code, dataset_index, len(dataset_codes))
-
-    logger.info("Indexing provider metadata...")
-    dbnomics_solr_client.index_provider(provider_metadata)
-    logger.debug("Committing Solr documents for provider metadata...")
-    dbnomics_solr_client.commit()
-    logger.info("Provider metadata indexed", provider_code=provider_code)
+        logger.info(
+            "Dataset %r (%d/%d) was indexed in %s",
+            dataset_code,
+            dataset_index,
+            len(dataset_codes),
+            format_timespan(time.time() - t0),
+        )
 
     if delete_obsolete_series:
         if successful_dataset_codes:
